@@ -112,7 +112,7 @@ pub mod rays {
             camera: &Camera,
             camera_transform: &GlobalTransform,
         ) -> Option<Self> {
-            let camera_position = camera_transform.compute_matrix();
+            let view = camera_transform.compute_matrix();
             let window = match windows.get(camera.window) {
                 Some(window) => window,
                 None => {
@@ -121,21 +121,26 @@ pub mod rays {
                 }
             };
             let screen_size = Vec2::from([window.width() as f32, window.height() as f32]);
-            let projection_matrix = camera.projection_matrix;
+            let projection = camera.projection_matrix;
+            let is_orthographic = projection.w_axis[3] == 1.0;
 
             // Normalized device coordinate cursor position from (-1, -1, -1) to (1, 1, 1)
             let cursor_ndc = (cursor_pos_screen / screen_size) * 2.0 - Vec2::from([1.0, 1.0]);
-            let cursor_pos_ndc_near: Vec3 = cursor_ndc.extend(-1.0);
-            let cursor_pos_ndc_far: Vec3 = cursor_ndc.extend(1.0);
+            let ndc_to_world: Mat4 = view * projection.inverse();
 
-            // Use near and far ndc points to generate a ray in world space This method is more
-            // robust than using the location of the camera as the start of the ray, because ortho
-            // cameras have a focal point at infinity!
-            let ndc_to_world: Mat4 = camera_position * projection_matrix.inverse();
-            let cursor_pos_near: Vec3 = ndc_to_world.project_point3(cursor_pos_ndc_near);
-            let cursor_pos_far: Vec3 = ndc_to_world.project_point3(cursor_pos_ndc_far);
-            let ray_direction = cursor_pos_far - cursor_pos_near;
-            Some(Ray3d::new(cursor_pos_near, ray_direction))
+            if is_orthographic {
+                let cursor_pos_ndc_near: Vec3 = cursor_ndc.extend(1.0);
+                let cursor_pos_ndc_far: Vec3 = cursor_ndc.extend(-1.0);
+                let cursor_pos_near: Vec3 = ndc_to_world.project_point3(cursor_pos_ndc_near);
+                let cursor_pos_far: Vec3 = ndc_to_world.project_point3(cursor_pos_ndc_far);
+                let ray_direction = (cursor_pos_far - cursor_pos_near).normalize();
+                Some(Ray3d::new(cursor_pos_near, ray_direction))
+            } else {
+                let cursor_pos_ndc_far: Vec3 = cursor_ndc.extend(1.0);
+                let cursor_pos_far: Vec3 = ndc_to_world.project_point3(cursor_pos_ndc_far);
+                let ray_direction = cursor_pos_far.normalize();
+                Some(Ray3d::new(camera_transform.translation, ray_direction))
+            }
         }
         /// Checks if the ray intersects with an AABB of a mesh.
         pub fn intersects_aabb(&self, aabb: &Aabb, model_to_world: &Mat4) -> Option<[f32; 2]> {
